﻿using DbContextLib;
using Entities;
using Microsoft.AspNetCore.Mvc.ModelBinding;
using Microsoft.EntityFrameworkCore;
using Model.OrderCriteria;
using Shared;

namespace DbDataManager.Service;

public class LessonDataService : ILessonService<LessonEntity>
{
    private UserDbContext DbContext { get; set; }

    public LessonDataService(UserDbContext context)
    {
        DbContext = context;
        context.Database.EnsureCreated();
    }

    public IEnumerable<LessonEntity> GetLessons(int page, int number, LessonOrderCriteria orderCriteria)
    {
        if (page <= 0)
        {
            page = 1;
        }
        if (number <= 0)
        {
            number = 10;
        }
        IQueryable<LessonEntity> query = DbContext.Lessons.Skip((page - 1) * number).Take(number);
        switch (orderCriteria)
        {
            case LessonOrderCriteria.None:
                break;
            case LessonOrderCriteria.ByTitle:
                query = query.OrderBy(s => s.Title);
                break;
            case LessonOrderCriteria.ByLastPublisher:
                query = query.OrderBy(s => s.LastPublisher);
                break;
            case LessonOrderCriteria.ByLastEdit:
                query = query.OrderBy(s => s.LastEdit);
                break;
            default:
                break;
        }

        var lessons = query.ToList();
        return lessons;
    }

    public LessonEntity GetLessonById(int id)
    {
        var lessonEntity = DbContext.Lessons.FirstOrDefault(u => u.Id == id);
        if (lessonEntity == null)
        {
            throw new ArgumentException("Impossible de trouver la leçon", nameof(id));
        }

        return lessonEntity;
    }

    public LessonEntity GetLessonByTitle(string title)
    {
        var lessonEntity = DbContext.Lessons.FirstOrDefault(u => u.Title == title);
        if (lessonEntity == null)
        {
            throw new ArgumentException("Impossible de trouver la leçon", nameof(title));
        }

        return lessonEntity;
    }

    public bool DeleteLesson(int id)
    {
        using (UserDbContext context = new UserDbContext(new DbContextOptions<UserDbContext>()))
        {
            var lessonEntity = context.Lessons.FirstOrDefault(l => l.Id == id);
            if (lessonEntity == null)
            {
                return false;
            }

            context.Lessons.Remove(lessonEntity);
            context.SaveChangesAsync();
            return true;
        }
    }

    public LessonEntity UpdateLesson(int id, LessonEntity lesson)
    {
        using (UserDbContext context = new UserDbContext(new DbContextOptions<UserDbContext>()))
        {
            var updatingLesson = context.Lessons.FirstOrDefault(l => l.Id == id);
            if (updatingLesson == null)
            {
                throw new ArgumentException("Impossible de trouver la leçon", nameof(id));
            }

            foreach (var pptt in typeof(LessonEntity).GetProperties()
                         .Where(p => p.CanWrite && p.Name != nameof(LessonEntity.Id)))
            {
                pptt.SetValue(updatingLesson, pptt.GetValue(lesson));
            }

            context.SaveChangesAsync();
            return updatingLesson;
        }
    }
    public LessonEntity CreateLesson(int id, string title, string lastPublisher, DateOnly lastEdit)
    {
        using (UserDbContext context = new UserDbContext(new DbContextOptions<UserDbContext>()))
        {
            DateTime date = DateTime.Now;
            var newLessonEntity = new LessonEntity()
            {
                Title = title,
                LastPublisher = lastPublisher,
                LastEdit = lastEdit,
            };
            var lesson = context.Lessons.FirstOrDefault(l => l.Id == id);
            if (lesson == null && id > 0)
            {
                newLessonEntity.Id = id;
                context.Lessons.Add(newLessonEntity);
                context.SaveChangesAsync();
                throw new ArgumentException(
                    $"Erreur, l'ID {id} est déjà utilisé pour une autre leçon, un id par a été attribué.");
            }

            context.Lessons.Add(newLessonEntity);
            context.SaveChangesAsync();
            return newLessonEntity;
        }
    }
}